@@ -23,7 +23,7 @@
 #define ARCH_INT64_TYPE long
 #define ARCH_UINT64_TYPE unsigned long
 #define ARCH_INT64_PRINTF_FORMAT "l"
-#undef ARCH_BIG_ENDIAN
+#define ARCH_BIG_ENDIAN
 #undef ARCH_ALIGN_DOUBLE
 #undef ARCH_ALIGN_INT64
 #undef NONSTANDARD_DIV_MOD
