@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: filename.mli,v 1.28 2002/04/18 09:11:18 guesdon Exp $ *)
+(* $Id: filename.mli,v 1.31.2.1 2005/01/31 17:01:02 doligez Exp $ *)
 
 (** Operations on file names. *)
 
@@ -48,10 +48,11 @@ val chop_suffix : string -> string -> string
 
 val chop_extension : string -> string
 (** Return the given file name without its extension. The extension
-   is the shortest suffix starting with a period, [.xyz] for instance.
+   is the shortest suffix starting with a period and not including
+   a directory separator, [.xyz] for instance.
 
    Raise [Invalid_argument] if the given name does not contain
-   a period. *)
+   an extension. *)
 
 val basename : string -> string
 (** Split a file name into directory name / base file name.
@@ -59,7 +60,10 @@ val basename : string -> string
    which is equivalent to [name]. Moreover, after setting the
    current directory to [dirname name] (with {!Sys.chdir}),
    references to [basename name] (which is a relative file name)
-   designate the same file as [name] before the call to {!Sys.chdir}. *)
+   designate the same file as [name] before the call to {!Sys.chdir}.
+
+   The result is not specified if the argument is not a valid file name
+   (for example, under Unix if there is a NUL character in the string). *)
 
 val dirname : string -> string
 (** See {!Filename.basename}. *)
@@ -76,11 +80,7 @@ val temp_file : string -> string -> string
    Under Unix, the temporary directory is [/tmp] by default; if set,
    the value of the environment variable [TMPDIR] is used instead.
    Under Windows, the name of the temporary directory is the
-   value of the environment variable [TEMP],
-   or [C:\temp] by default.
-   Under MacOS, the name of the temporary directory is given
-   by the environment variable [TempFolder]; if not set,
-   temporary files are created in the current directory. *)
+   value of the environment variable [TEMP], or [C:\temp] by default. *)
 
 val open_temp_file :
       ?mode: open_flag list -> string -> string -> string * out_channel
