@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: int32.mli,v 1.12 2002/06/26 14:52:34 xleroy Exp $ *)
+(* $Id: int32.mli,v 1.16.6.1 2005/04/11 16:51:42 doligez Exp $ *)
 
 (** 32-bit integers.
 
@@ -48,14 +48,13 @@ external mul : int32 -> int32 -> int32 = "%int32_mul"
 (** Multiplication. *)
 
 external div : int32 -> int32 -> int32 = "%int32_div"
-(** Integer division.  Raise [Division_by_zero] if the second 
+(** Integer division.  Raise [Division_by_zero] if the second
    argument is zero.  This division rounds the real quotient of
    its arguments towards zero, as specified for {!Pervasives.(/)}. *)
 
 external rem : int32 -> int32 -> int32 = "%int32_mod"
 (** Integer remainder.  If [y] is not zero, the result
-   of [Int32.rem x y] satisfies the following properties:
-   [Int32.zero <= Int32.rem x y < Int32.abs y] and
+   of [Int32.rem x y] satisfies the following property:
    [x = Int32.add (Int32.mul (Int32.div x y) y) (Int32.rem x y)].
    If [y = 0], [Int32.rem x y] raises [Division_by_zero]. *)
 
@@ -104,7 +103,8 @@ external shift_right_logical : int32 -> int -> int32 = "%int32_lsr"
    The result is unspecified if [y < 0] or [y >= 32]. *)
 
 external of_int : int -> int32 = "%int32_of_int"
-(** Convert the given integer (type [int]) to a 32-bit integer (type [int32]). *)
+(** Convert the given integer (type [int]) to a 32-bit integer
+    (type [int32]). *)
 
 external to_int : int32 -> int = "%int32_to_int"
 (** Convert the given 32-bit integer (type [int32]) to an
@@ -113,33 +113,38 @@ external to_int : int32 -> int = "%int32_to_int"
    during the conversion.  On 64-bit platforms, the conversion
    is exact. *)
 
-external of_float : float -> int32 = "int32_of_float"
+external of_float : float -> int32 = "caml_int32_of_float"
 (** Convert the given floating-point number to a 32-bit integer,
    discarding the fractional part (truncate towards 0).
    The result of the conversion is undefined if, after truncation,
    the number is outside the range \[{!Int32.min_int}, {!Int32.max_int}\]. *)
 
-external to_float : int32 -> float = "int32_to_float"
+external to_float : int32 -> float = "caml_int32_to_float"
 (** Convert the given 32-bit integer to a floating-point number. *)
 
-external of_string : string -> int32 = "int32_of_string"
+external of_string : string -> int32 = "caml_int32_of_string"
 (** Convert the given string to a 32-bit integer.
    The string is read in decimal (by default) or in hexadecimal,
    octal or binary if the string begins with [0x], [0o] or [0b]
    respectively.
    Raise [Failure "int_of_string"] if the given string is not
-   a valid representation of an integer. *)
+   a valid representation of an integer, or if the integer represented
+   exceeds the range of integers representable in type [int32]. *)
 
 val to_string : int32 -> string
 (** Return the string representation of its argument, in signed decimal. *)
 
-external format : string -> int32 -> string = "int32_format"
-(** [Int32.format fmt n] return the string representation of the
-   32-bit integer [n] in the format specified by [fmt].
-   [fmt] is a [Printf]-style format containing exactly
-   one [%d], [%i], [%u], [%x], [%X] or [%o] conversion specification.
-   This function is deprecated; use {!Printf.sprintf} with a [%lx] format
-   instead. *)
+external bits_of_float : float -> int32 = "caml_int32_bits_of_float"
+(** Return the internal representation of the given float according
+   to the IEEE 754 floating-point ``single format'' bit layout.
+   Bit 31 of the result represents the sign of the float;
+   bits 30 to 23 represent the (biased) exponent; bits 22 to 0
+   represent the mantissa. *)
+
+external float_of_bits : int32 -> float = "caml_int32_float_of_bits"
+(** Return the floating-point number whose internal representation,
+   according to the IEEE 754 floating-point ``single format'' bit layout,
+   is the given [int32]. *)
 
 type t = int32
 (** An alias for the type of 32-bit integers. *)
@@ -149,3 +154,16 @@ val compare: t -> t -> int
     {!Pervasives.compare}.  Along with the type [t], this function [compare]
     allows the module [Int32] to be passed as argument to the functors
     {!Set.Make} and {!Map.Make}. *)
+
+(**/**)
+
+(** {6 Deprecated functions} *)
+
+external format : string -> int32 -> string = "caml_int32_format"
+(** [Int32.format fmt n] return the string representation of the
+   32-bit integer [n] in the format specified by [fmt].
+   [fmt] is a [Printf]-style format consisting of exactly
+   one [%d], [%i], [%u], [%x], [%X] or [%o] conversion specification.
+   This function is deprecated; use {!Printf.sprintf} with a [%lx] format
+   instead. *)
+
