@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: list.ml,v 1.29 2002/06/05 12:12:46 doligez Exp $ *)
+(* $Id: list.ml,v 1.31 2004/01/01 16:42:40 doligez Exp $ *)
 
 (* List operations *)
 
@@ -134,7 +134,7 @@ let rec exists2 p l1 l2 =
 
 let rec mem x = function
     [] -> false
-  | a::l -> a = x || mem x l
+  | a::l -> compare a x = 0 || mem x l
 
 let rec memq x = function
     [] -> false
@@ -142,7 +142,7 @@ let rec memq x = function
 
 let rec assoc x = function
     [] -> raise Not_found
-  | (a,b)::l -> if a = x then b else assoc x l
+  | (a,b)::l -> if compare a x = 0 then b else assoc x l
 
 let rec assq x = function
     [] -> raise Not_found
@@ -150,7 +150,7 @@ let rec assq x = function
 
 let rec mem_assoc x = function
   | [] -> false
-  | (a, b) :: l -> a = x || mem_assoc x l
+  | (a, b) :: l -> compare a x = 0 || mem_assoc x l
 
 let rec mem_assq x = function
   | [] -> false
@@ -158,7 +158,7 @@ let rec mem_assq x = function
 
 let rec remove_assoc x = function
   | [] -> []
-  | (a, b as pair) :: l -> if a = x then l else pair :: remove_assoc x l
+  | (a, b as pair) :: l -> if compare a x = 0 then l else pair :: remove_assoc x l
 
 let rec remove_assq x = function
   | [] -> []
@@ -290,7 +290,7 @@ let fast_sort = stable_sort;;
    Also, there seems to be a bug in this code or in the
    implementation of obj_truncate.
 
-external obj_truncate : 'a array -> int -> unit = "obj_truncate"
+external obj_truncate : 'a array -> int -> unit = "caml_obj_truncate"
 
 let array_to_list_in_place a =
   let l = Array.length a in
