@@ -11,28 +11,29 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: marshal.ml,v 1.7 2002/07/12 09:47:54 xleroy Exp $ *)
+(* $Id: marshal.ml,v 1.8 2004/01/01 16:42:40 doligez Exp $ *)
 
 type extern_flags =
     No_sharing
   | Closures
 
 external to_channel: out_channel -> 'a -> extern_flags list -> unit
-    = "output_value"
+    = "caml_output_value"
 external to_string: 'a -> extern_flags list -> string
-    = "output_value_to_string"
+    = "caml_output_value_to_string"
 external to_buffer_unsafe:
       string -> int -> int -> 'a -> extern_flags list -> int
-    = "output_value_to_buffer"
+    = "caml_output_value_to_buffer"
 
 let to_buffer buff ofs len v flags =
   if ofs < 0 || len < 0 || ofs > String.length buff - len
   then invalid_arg "Marshal.to_buffer: substring out of bounds"
   else to_buffer_unsafe buff ofs len v flags
 
-external from_channel: in_channel -> 'a = "input_value"
-external from_string_unsafe: string -> int -> 'a = "input_value_from_string"
-external data_size_unsafe: string -> int -> int = "marshal_data_size"
+external from_channel: in_channel -> 'a = "caml_input_value"
+external from_string_unsafe: string -> int -> 'a
+                           = "caml_input_value_from_string"
+external data_size_unsafe: string -> int -> int = "caml_marshal_data_size"
 
 let header_size = 20
 let data_size buff ofs =
